OrbPalette class

Predefined color palettes for the Orb

Constructors

OrbPalette()

Properties

hashCode int
The hash code for this object.
no setterinherited
runtimeType Type
A representation of the runtime type of the object.
no setterinherited

Methods

noSuchMethod(Invocation invocation) → dynamic
Invoked when a nonexistent method or property is accessed.
inherited
toString() String
A string representation of this object.
inherited

Operators

operator ==(Object other) bool
The equality operator.
inherited

Constants

aurora → const List<Color>
inferno → const List<Color>
siriOriginal → const List<Color>